@@ -1,4 +1,6 @@
-import tempfile
+import logging
+
+import plistlib
 
 import click
 
@@ -6,7 +8,8 @@ from pymobiledevice3.cli.cli_common import Command, print_json, CommandWithoutAu
 from pymobiledevice3.exceptions import PasscodeRequiredError
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.services.heartbeat import HeartbeatService
-from pymobiledevice3.tcp_forwarder import TcpForwarder
+
+logger = logging.getLogger(__name__)
 
 
 @click.group()
@@ -21,27 +24,6 @@ def lockdown_group():
     pass
 
 
-@lockdown_group.command('forward', cls=Command)
-@click.argument('src_port', type=click.IntRange(1, 0xffff))
-@click.argument('dst_port', type=click.IntRange(1, 0xffff))
-@click.option('-d', '--daemonize', is_flag=True)
-def lockdown_forward(lockdown: LockdownClient, src_port, dst_port, daemonize):
-    """ forward tcp port """
-    forwarder = TcpForwarder(lockdown, src_port, dst_port)
-
-    if daemonize:
-        try:
-            from daemonize import Daemonize
-        except ImportError:
-            raise NotImplementedError('daemonizing is only supported on unix platforms')
-
-        with tempfile.NamedTemporaryFile('wt') as pid_file:
-            daemon = Daemonize(app=f'forwarder {src_port}->{dst_port}', pid=pid_file.name, action=forwarder.start)
-            daemon.start()
-    else:
-        forwarder.start()
-
-
 @lockdown_group.command('recovery', cls=Command)
 def lockdown_recovery(lockdown: LockdownClient):
     """ enter recovery """
@@ -103,6 +85,16 @@ def lockdown_pair(lockdown: LockdownClient):
     lockdown.pair()
 
 
+@lockdown_group.command('save-pair-record', cls=CommandWithoutAutopair)
+@click.argument('output', type=click.File('wb'))
+def lockdown_save_pair_record(lockdown: LockdownClient, output):
+    """ save pair record to specified location """
+    if lockdown.pair_record is None:
+        logger.error('no pairing record was found')
+        return
+    plistlib.dump(lockdown.pair_record, output)
+
+
 @lockdown_group.command('date', cls=Command)
 def lockdown_date(lockdown: LockdownClient):
     """ get device date """
