@@ -1,7 +1,9 @@
+import asn1
 import logging
+import struct
 from typing import Optional
 
-import asn1
+from pymobiledevice3.exceptions import PyMobileDevice3Exception
 
 logger = logging.getLogger(__name__)
 
@@ -152,9 +154,11 @@ def asn1_find_element(index: int, type_: int, data: bytes) -> Optional[int]:
     return off
 
 
-def stitch_component(name: str, data: bytes, blob: bytes):
+def stitch_component(name: str, data: bytes, tss):
     logger.info(f'Personalizing IMG4 component {name}...')
 
+    blob = tss.ap_img4_ticket
+
     # first we need check if we have to change the tag for the given component
     tag_off = asn1_find_element(1, asn1.Numbers.IA5String, data)
 
@@ -204,6 +208,31 @@ def stitch_component(name: str, data: bytes, blob: bytes):
 
     encoder.leave()
 
+    # hack if we have a *-TBM entry for the give
+    tbm_dict = tss.get(f'{name}-TBM')
+    if tbm_dict is not None:
+        # now construct IM4R
+        encoder.enter(asn1.Numbers.Boolean, cls=asn1.Classes.Context)
+        encoder.enter(asn1.Numbers.Sequence)
+        encoder.write(b'IM4R', asn1.Numbers.IA5String)
+        encoder.enter(asn1.Numbers.Set)
+
+        for tbm_component in (b'ucon', b'ucer'):
+            # write priv ucon element
+            encoder.enter(struct.unpack('>I', tbm_component)[0], cls=asn1.Classes.Private)
+
+            # write ucon IA5STRING and ucon data
+            encoder.enter(asn1.Numbers.Sequence)
+            encoder.write(tbm_component, asn1.Numbers.IA5String)
+            encoder.write(tbm_dict[tbm_component.decode()])
+            encoder.leave()
+
+            encoder.leave()
+
+        encoder.leave()
+        encoder.leave()
+        encoder.leave()
+
     encoder.leave()
 
     return encoder.output()
