@@ -350,7 +350,16 @@ class TSSRequest:
                     k = 'SepNonce'
                 self._request[k] = v
 
+        uid_mode = parameters.get('UID_MODE')
+        requires_uid_mode = parameters.get('RequiresUIDMode')
+        if uid_mode is not None:
+            self._request['UID_MODE'] = uid_mode
+        elif requires_uid_mode is not None:
+            # The logic here is missing why this value is expected to be 'false'
+            self._request['UID_MODE'] = False
+
         self._request['@ApImg4Ticket'] = True
+        self._request['@BBTicket'] = True
 
     def add_se_tags(self, parameters: typing.Mapping, overrides=None):
         manifest = parameters['Manifest']
@@ -739,7 +748,11 @@ class TSSRequest:
 
         return p.output()
 
-    def update(self, options):
+    def remove_key(self, key: str) -> None:
+        if key in self._request:
+            self._request.pop(key)
+
+    def update(self, options) -> None:
         self._request.update(options)
 
     def send_receive(self) -> TSSResponse:
